module Gargantext.Prelude.Error (
  -- * Attaching callstacks to exceptions
    WithStacktrace(..)
  , UnexpectedPanic(..)
  , withStacktrace

  -- * Drop-in replacement for panic/error
  , panicTrace
  , errorTrace
  ) where

import Control.Exception
import Data.Text qualified as T
import GHC.Stack
import Prelude

-- | A 'WithStacktrace' carries an error alongside its
-- 'CallStack', to be able to print the correct source location
-- of where the error originated.
data WithStacktrace e =
  WithStacktrace {
    ct_callStack :: !CallStack
  , ct_error     :: !e
  } deriving Show

instance Exception e => Exception (WithStacktrace e) where
  displayException WithStacktrace{..}
    = displayException ct_error <> "\n" <> prettyCallStack ct_callStack

withStacktrace :: HasCallStack => e -> WithStacktrace e
withStacktrace = withFrozenCallStack . WithStacktrace callStack

newtype UnexpectedPanic = UnexpectedPanic T.Text
  deriving Show

instance Exception UnexpectedPanic

panicTrace :: HasCallStack => T.Text -> x
panicTrace = throw . withFrozenCallStack . WithStacktrace callStack . UnexpectedPanic

-- | Drop-in replacement for Prelude's 'error'.
errorTrace :: HasCallStack => String -> x
errorTrace = panicTrace . T.pack
